#ifndef TANGO_COMPILATION_SESSION_HPP
#define TANGO_COMPILATION_SESSION_HPP

#include "tango/local_logger.hpp"
#include <chrono>

//-------------------------------------------------------------------------------------//

namespace tango {

//-------------------------------------------------------------------------------------//

class CompilationSession;

extern thread_local CompilationSession session;

//-------------------------------------------------------------------------------------//
// Both enums require for resolve compilation mode and which compiler is used
enum class CompMode : uint8_t {
  Stub,
  Regular,
  OSR,
  Native
};

enum class CompType : uint8_t {
  Tango,
  Opto
};

typedef arena_vector<PhaseName, persistent_allocator>::type PhaseStack;
using TimePoint = std::chrono::time_point<std::chrono::system_clock>;

//-------------------------------------------------------------------------------------//
// Contain information about current compilation, mostly required for logger & verifier
// also need for exclude direct using of LocalLogger
class CompilationSession {
  friend class PassManager;
  friend class LocalLogger;
 public:
  CompilationSession();

  static void     openSession(Ir*,size_t, CompType, CompMode, int, 
                              const char*, const char*, Options* = nullptr);
  static void     exitSession(const char*);
  static void     closeSession();

  void            startPhase(PhaseName);
  void            endPhase();

  bool            isPassed(uint32_t) const;
  bool            isAfterSelector()  const  { return _passed_phases == BIT(static_cast<uint32_t>(PhaseName::SEL)); }
  bool            isNested()         const  { return _phases.size() > 1; }

  PhaseName       getCurrent()       const  { return _name;            }
  size_t          getPhaseMask()     const  { return _passed_phases;   }

 private:
  void            dumpSession()      const; // for debugging purpose

 public:
  Ir*             _ir;
  
 private:
  PhaseName       _name;          // current active phase
  PhaseStack      _phases;

  size_t          _session_id;    // id os session generate by PassManager
  size_t          _compile_id;    // id of compilation, get from compiler or same as session
  uint32_t        _passed_phases; // phase which was finished, required for verifier

  int             _bci_entry;

  CompType        _comp_type;
  CompMode        _comp_mode;

  TimePoint       _start_time;

  Options*        _log_opts;     // logging options for this session
};


//-------------------------------------------------------------------------------------//

}      // namespace tango
#endif // COMPILATION_SESSION_HPP

